
from unittest import TestCase

from lily_env.validator import url, length, ValidatorError


class UrlTestCase(TestCase):

    def test_success(self):
        assert url('Y', 'http://hi.io') is True

    def test_not_valid(self):

        try:
            url('Y', 'hi.io')

        except ValidatorError as e:
            assert e.args[0] == 'env.Y: Text "hi.io" is not valid URL'

        else:
            raise AssertionError('should raise error')


class LengthTestCase(TestCase):

    def test_success(self):
        assert length('Y', 'hello world') is True

    def test_not_valid(self):

        try:
            length('Y', 'hello', min_length=10)

        except ValidatorError as e:
            assert e.args[0] == (
                'env.Y: Text "hello" is too short. min length: 10')

        else:
            raise AssertionError('should raise error')




from unittest import TestCase

from lily_env.fields import (
    BooleanField,
    CharField,
    FloatField,
    IntegerField,
    URLField,
)
from lily_env.validator import ValidatorError


class BooleanFieldTestCase(TestCase):

    def test_to_python(self):

        assert BooleanField().to_python('X', 'True') is True
        assert BooleanField().to_python('X', 'TRUE') is True
        assert BooleanField().to_python('X', 'False') is False
        assert BooleanField().to_python('X', 'FALse') is False

    def test_to_python__allow_null(self):

        f = BooleanField(allow_null=True)

        assert f.to_python('X', None) is None

    def test_to_python__not_allow_null(self):

        f = BooleanField(allow_null=False)

        try:
            f.to_python('X', None)

        except ValidatorError as e:
            assert e.args[0] == 'env.X: Null value are not allowed'

        else:
            raise AssertionError('should raise error')

    def test_to_python__not_valid(self):

        f = BooleanField()

        try:
            f.to_python('X', 'WHAT')

        except ValidatorError as e:
            assert e.args[0] == 'env.X: Cannot cast WHAT to boolean'

        else:
            raise AssertionError('should raise error')


class CharFieldTestCase(TestCase):

    def test_to_python(self):

        assert CharField().to_python('X', 'Hi there') == 'Hi there'
        assert CharField().to_python('X', 'cześć') == 'cześć'

    def test_to_python__allow_null(self):

        f = CharField(allow_null=True)

        assert f.to_python('X', None) is None

    def test_to_python__not_allow_null(self):

        f = CharField(allow_null=False)

        try:
            f.to_python('X', None)

        except ValidatorError as e:
            assert e.args[0] == 'env.X: Null value are not allowed'

        else:
            raise AssertionError('should raise error')

    def test_to_python__not_valid(self):

        cases = [
            (
                CharField(min_length=2),
                'h',
                'env.X: Text "h" is too short. min length: 2',
            ),
            (
                CharField(max_length=4),
                'hello',
                'env.X: Text "hello" is too long. max length: 4',
            ),
            (
                CharField(min_length=3, max_length=4),
                'hi',
                'env.X: Text "hi" is too short. min length: 3',
            ),
        ]

        for f, text, message in cases:
            try:
                f.to_python('X', text)

            except ValidatorError as e:
                assert e.args[0] == message

            else:
                raise AssertionError('should raise error')


class FloatFieldTestCase(TestCase):

    def test_to_python(self):

        assert FloatField().to_python('X', '12') == 12.0
        assert FloatField().to_python('X', '14.5') == 14.5

    def test_to_python__allow_null(self):

        f = FloatField(allow_null=True)

        assert f.to_python('X', None) is None

    def test_to_python__not_allow_null(self):

        f = FloatField(allow_null=False)

        try:
            f.to_python('X', None)

        except ValidatorError as e:
            assert e.args[0] == 'env.X: Null value are not allowed'

        else:
            raise AssertionError('should raise error')

    def test_to_python__not_valid(self):

        f = FloatField()

        try:
            f.to_python('X', 'WHAT')

        except ValidatorError as e:
            assert e.args[0] == 'env.X: Cannot cast WHAT to float'

        else:
            raise AssertionError('should raise error')


class IntegerFieldTestCase(TestCase):

    def test_to_python(self):

        assert IntegerField().to_python('X', '12') == 12
        assert IntegerField().to_python('X', '14.5') == 14

    def test_to_python__allow_null(self):

        f = IntegerField(allow_null=True)

        assert f.to_python('X', None) is None

    def test_to_python__not_allow_null(self):

        f = IntegerField(allow_null=False)

        try:
            f.to_python('X', None)

        except ValidatorError as e:
            assert e.args[0] == 'env.X: Null value are not allowed'

        else:
            raise AssertionError('should raise error')

    def test_to_python__not_valid(self):

        f = IntegerField()

        try:
            f.to_python('X', 'WHAT')

        except ValidatorError as e:
            assert e.args[0] == 'env.X: Cannot cast WHAT to integer'

        else:
            raise AssertionError('should raise error')


class URLFieldTestCase(TestCase):

    def test_to_python(self):

        assert URLField().to_python(
            'X', 'http://127.0.0.1') == 'http://127.0.0.1'
        assert URLField().to_python(
            'X', 'ws://hi.org:123') == 'ws://hi.org:123'
        assert URLField().to_python(
            'X', 'ftp://hi.org') == 'ftp://hi.org'

    def test_to_python__allow_null(self):

        f = URLField(allow_null=True)

        assert f.to_python('X', None) is None

    def test_to_python__not_allow_null(self):

        f = URLField(allow_null=False)

        try:
            f.to_python('X', None)

        except ValidatorError as e:
            assert e.args[0] == 'env.X: Null value are not allowed'

        else:
            raise AssertionError('should raise error')

    def test_to_python__not_valid(self):

        f = URLField()

        try:
            f.to_python('X', 'WHAT')

        except ValidatorError as e:
            assert e.args[0] == 'env.X: Text "WHAT" is not valid URL'

        else:
            raise AssertionError('should raise error')




import json
import os

from .fields import BaseField


class Env:
    """
    Class which captures all already validated and discovered environment
    variables.

    """

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

        with open(Env.get_dump_filepath(), 'w') as f:
            f.write(json.dumps(kwargs))

    @classmethod
    def from_dump(cls):
        with open(Env.get_dump_filepath(), 'r') as f:
            return json.loads(f.read())

    @staticmethod
    def get_dump_filepath():
        base_dir = os.path.dirname(__file__)
        return f'{base_dir}/lily_env_dump.json'


class EnvParser:

    def __init__(self):

        env_variables = {}
        for field_name, field in self.fields.items():
            if field.required:
                raw_value = os.environ[field_name.upper()]

            else:
                raw_value = os.environ.get(
                    field_name.upper(), field.default)

            env_variables[field_name] = field.to_python(field_name, raw_value)

        self.env_variables = env_variables

    @property
    def fields(self):
        fields = {}
        for name in dir(self):
            if name != 'fields':
                attr = getattr(self, name)
                if isinstance(attr, BaseField):
                    fields[name] = attr

        return fields

    def parse(self):
        return Env(**self.env_variables)




import click

from .parser import Env


@click.group()
def cli():
    """Expose multiple commands allowing one to work with lily_env."""
    pass


@click.command()
def dump():
    """
    Dump currently loaded env variables into a console.

    """

    try:
        click.echo('\n'.join([
            f'{k.upper()}: {v}' for k, v in Env.from_dump().items()]))

    except FileNotFoundError:
        raise click.ClickException(
            'Dump file does not exist, run `parse` on your '
            '`EnvParser` instance in order to render it')


cli.add_command(dump)




from . import validator as val


class BaseField:

    def __init__(
            self,
            required=True,
            default=None,
            allow_null=False,
            description=None):

        self.required = required
        self.default = default
        self.allow_null = allow_null
        self.description = description

    def to_python(self, field_name, value):

        if not self.allow_null:
            val.not_null(field_name, value)

        elif value is None and self.allow_null:
            return val.NULL


class BooleanField(BaseField):

    def to_python(self, field_name, value):

        # -- base validation
        result = super(BooleanField, self).to_python(field_name, value)
        if result == val.NULL:
            return None

        if isinstance(value, bool):
            return value

        if value.upper() == 'TRUE':
            return True

        elif value.upper() == 'FALSE':
            return False

        else:
            raise val.ValidatorError(
                f'env.{field_name}: Cannot cast {value} to boolean')


class CharField(BaseField):

    def __init__(self, min_length=None, max_length=None, **kwargs):
        self.min_length = min_length
        self.max_length = max_length
        super(CharField, self).__init__(**kwargs)

    def to_python(self, field_name, value):

        # -- base validation
        result = super(CharField, self).to_python(field_name, value)
        if result == val.NULL:
            return None

        val.length(
            field_name,
            value,
            min_length=self.min_length,
            max_length=self.max_length)

        return value


class FloatField(BaseField):

    def to_python(self, field_name, value):

        # -- base validation
        result = super(FloatField, self).to_python(field_name, value)
        if result == val.NULL:
            return None

        try:
            return float(value)

        except ValueError:
            raise val.ValidatorError(
                f'env.{field_name}: Cannot cast {value} to float')


class IntegerField(BaseField):

    def to_python(self, field_name, value):

        # -- base validation
        result = super(IntegerField, self).to_python(field_name, value)
        if result == val.NULL:
            return None

        try:
            return int(float(value))

        except ValueError:
            raise val.ValidatorError(
                f'env.{field_name}: Cannot cast {value} to integer')


class URLField(BaseField):

    def to_python(self, field_name, value):

        # -- base validation
        result = super(URLField, self).to_python(field_name, value)
        if result == val.NULL:
            return None

        val.url(field_name, value)

        return value




import re

# -- import all external validators, some of them will be overloaded
# -- with the custom implementations
import validators as vals  # noqa


NULL = 'NULL'


class ValidatorError(Exception):
    pass


def not_null(field_name, value):

    message = f'env.{field_name}: Null value are not allowed'

    if value is None:
        raise ValidatorError(message)


def url(field_name, url_text):

    message = f'env.{field_name}: Text "{url_text}" is not valid URL'

    # -- in order to force the underlying library to work correctly
    # -- with websockets protocol as well as some other future protocols
    # -- the following trick must be included
    modified_url_text = re.sub(r'^wss?', 'http', url_text)
    if vals.url(modified_url_text) is not True:
        raise ValidatorError(message)

    return True


def length(field_name, text, min_length=None, max_length=None):

    if min_length:
        message = (
            f'env.{field_name}: Text "{text}" is too short. '
            f'min length: {min_length}')

        if vals.length(text, min=min_length) is not True:
            raise ValidatorError(message)

    if max_length:
        message = (
            f'env.{field_name}: Text "{text}" is too long. '
            f'max length: {max_length}')

        if vals.length(text, max=max_length) is not True:
            raise ValidatorError(message)

    return True




import os
from unittest import TestCase

import pytest

import lily_env as env
from lily_env.parser import Env


class EnvTestCase(TestCase):

    @pytest.fixture(autouse=True)
    def initfixtures(self, mocker, tmpdir):
        self.mocker = mocker
        self.tmpdir = tmpdir

    #
    # __INIT__
    #
    def test__init__(self):

        self.tmpdir.join('lily_env_dump.json').write('hey')
        self.mocker.patch.object(
            Env,
            'get_dump_filepath'
        ).return_value = str(self.tmpdir.join('lily_env_dump.json'))

        e = Env(is_prod=True, secret_key='hello')

        assert e.is_prod is True
        assert e.secret_key == 'hello'
        assert self.tmpdir.join('lily_env_dump.json').read() == (
            '{"is_prod": true, "secret_key": "hello"}')

    #
    # FROM_DUMP
    #
    def test_from_dump(self):

        self.tmpdir.join('lily_env_dump.json').write(
            '{"is_prod": false, "secret_key": "hello"}')
        self.mocker.patch.object(
            Env,
            'get_dump_filepath'
        ).return_value = str(self.tmpdir.join('lily_env_dump.json'))

        assert Env.from_dump() == {"is_prod": False, "secret_key": "hello"}


class EnvParserTestCase(TestCase):

    def setUp(self):

        try:
            del os.environ['SECRET_KEY']
            del os.environ['IS_IMPORTANT']
            del os.environ['AWS_URL']
            del os.environ['NUMBER_OF_WORKERS']

        except KeyError:
            pass

    def test_parse(self):

        class MyEnvParser(env.EnvParser):

            secret_key = env.CharField()

            is_important = env.BooleanField()

            aws_url = env.URLField()

            number_of_workers = env.IntegerField()

        os.environ['SECRET_KEY'] = 'secret.whatever'
        os.environ['IS_IMPORTANT'] = 'true'
        os.environ['AWS_URL'] = 'http://hello.word.org'
        os.environ['NUMBER_OF_WORKERS'] = '113'

        e = MyEnvParser().parse()

        assert e.secret_key == 'secret.whatever'
        assert e.is_important is True
        assert e.aws_url == 'http://hello.word.org'
        assert e.number_of_workers == 113

    def test_parse__optional_with_defaults(self):

        class MyEnvParser(env.EnvParser):

            secret_key = env.CharField(required=False, default='hello')

            is_important = env.BooleanField(required=False, default=False)

            aws_url = env.URLField(required=False, default='http://hi.pl')

            number_of_workers = env.IntegerField(required=False, default=12)

        e = MyEnvParser().parse()

        assert e.secret_key == 'hello'
        assert e.is_important is False
        assert e.aws_url == 'http://hi.pl'
        assert e.number_of_workers == 12

    def test_parse__optional_without_defaults(self):

        class MyEnvParser(env.EnvParser):

            secret_key = env.CharField(required=False, allow_null=True)

            is_important = env.BooleanField(required=False, allow_null=True)

            aws_url = env.URLField(required=False, allow_null=True)

            number_of_workers = env.IntegerField(
                required=False, allow_null=True)

        e = MyEnvParser().parse()

        assert e.secret_key is None
        assert e.is_important is None
        assert e.aws_url is None
        assert e.number_of_workers is None

    def test_parse__validation_errors(self):

        class MyEnvParser(env.EnvParser):

            secret_key = env.CharField(max_length=12)

            is_important = env.BooleanField()

            aws_url = env.URLField()

            number_of_workers = env.IntegerField()

        os.environ['SECRET_KEY'] = 'secret.whatever'
        os.environ['IS_IMPORTANT'] = 'whatever'
        os.environ['AWS_URL'] = 'not.url'
        os.environ['NUMBER_OF_WORKERS'] = 'not.number'

        try:
            MyEnvParser().parse()

        except env.ValidatorError as e:
            assert e.args[0] == (
                'env.aws_url: Text "not.url" is not valid URL')

        else:
            raise AssertionError('should raise exception')




import os
from unittest import TestCase
import textwrap

from click.testing import CliRunner
import pytest

import lily_env as env
from lily_env.cli import cli
from lily_env.parser import Env


class CliTestCase(TestCase):

    @pytest.fixture(autouse=True)
    def initfixtures(self, mocker, tmpdir):
        self.mocker = mocker
        self.tmpdir = tmpdir

    def setUp(self):
        self.runner = CliRunner()

    #
    # DUMP
    #
    def test_dump(self):

        class MyEnvParser(env.EnvParser):

            secret_key = env.CharField()

            is_important = env.BooleanField()

            aws_url = env.URLField()

            number_of_workers = env.IntegerField()

        os.environ['SECRET_KEY'] = 'secret.whatever'
        os.environ['IS_IMPORTANT'] = 'true'
        os.environ['AWS_URL'] = 'http://hello.word.org'
        os.environ['NUMBER_OF_WORKERS'] = '113'

        self.tmpdir.join('lily_env_dump.json').write('hey')
        self.mocker.patch.object(
            Env,
            'get_dump_filepath'
        ).return_value = str(self.tmpdir.join('lily_env_dump.json'))

        MyEnvParser().parse()

        result = self.runner.invoke(cli, ['dump'])

        assert result.exit_code == 0
        assert result.output.strip() == textwrap.dedent('''
            AWS_URL: http://hello.word.org
            IS_IMPORTANT: True
            NUMBER_OF_WORKERS: 113
            SECRET_KEY: secret.whatever
        ''').strip()

    def test_dump__dump_was_not_created(self):

        class MyEnvParser(env.EnvParser):

            secret_key = env.CharField()

        os.environ['SECRET_KEY'] = 'secret.whatever'

        self.mocker.patch.object(
            Env,
            'get_dump_filepath'
        ).return_value = str(self.tmpdir.join('lily_env_dump.json'))

        MyEnvParser()

        result = self.runner.invoke(cli, ['dump'])

        assert result.exit_code == 1
        assert result.output.strip() == (
            'Error: Dump file does not exist, run `parse` on your '
            '`EnvParser` instance in order to render it')



#!/usr/bin/env python

import os.path
from setuptools import setup, find_packages


requirements_path = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), 'requirements.txt')


setup(
    name='lily-env',
    description=(
        'Lily extension for managing environment variables, '
        'their validity etc.'),
    version='0.0.2',
    author='CoSphere Tech Team',
    url='https://bitbucket.org/goodai/lily-env',
    packages=find_packages(),
    entry_points='''
        [console_scripts]
        lily_env=lily_env.cli:cli
    ''',
    install_requires=open(requirements_path).readlines(),
    package_data={'': ['requirements.txt']},
    include_package_data=True)
